@@ -46,9 +46,10 @@ from distutils.version import LooseVersion
 import easybuild.tools.environment as env
 from easybuild.framework.easyconfig import EasyConfig, get_paths_for
 from easybuild.tools.build_log import EasyBuildError, init_logger, print_msg, remove_log_handler
-from easybuild.tools.config import build_path, install_path, log_path, read_only_installdir, source_path
+from easybuild.tools.config import build_path, install_path, log_path, read_only_installdir
+from easybuild.tools.config import source_path, module_classes
 from easybuild.tools.filetools import adjust_permissions, apply_patch, convert_name, download_file
-from easybuild.tools.filetools import encode_class_name, extract_file, run_cmd, rmtree2
+from easybuild.tools.filetools import encode_class_name, extract_file, run_cmd, rmtree2, modify_env
 from easybuild.tools.module_generator import GENERAL_CLASS, ModuleGenerator
 from easybuild.tools.modules import Modules, get_software_root
 from easybuild.tools.systemtools import get_core_count
@@ -75,7 +76,7 @@ class EasyBlock(object):
     #
     # INIT
     #
-    def __init__(self, path, debug=False):
+    def __init__(self, path, debug=False, robot_path=None):
         """
         Initialize the EasyBlock instance.
         """
@@ -92,11 +93,12 @@ class EasyBlock(object):
         self.exts = None
         self.ext_instances = []
         self.skip = None
-        self.module_extra_extensions = ''  # extra stuff for module file required by extentions
+        self.module_extra_extensions = ''  # extra stuff for module file required by extensions
 
         # easyconfig for this application
         all_stops = [x[0] for x in self.get_steps()]
-        self.cfg = EasyConfig(path, self.extra_options(), valid_stops=all_stops)
+        self.cfg = EasyConfig(path, extra_options=self.extra_options(), valid_module_classes=module_classes(),
+                              valid_stops=all_stops)
 
         # module generator
         self.moduleGenerator = None
@@ -120,6 +122,12 @@ class EasyBlock(object):
         # list of loaded modules
         self.loaded_modules = []
 
+        # robot path
+        self.robot_path = robot_path
+
+        # original module path
+        self.orig_modulepath = os.getenv('MODULEPATH')
+
     # INIT/CLOSE LOG
     def init_log(self):
         """
@@ -149,12 +157,19 @@ class EasyBlock(object):
         All source files will be checked if a file exists (or can be located)
         """
 
-        for source in list_of_sources:
-            ## check if the sources can be located
+        for src_entry in list_of_sources:
+            if type(src_entry) in [list, tuple]:
+                cmd = src_entry[1]
+                source = src_entry[0]
+            elif type(src_entry) == str:
+                cmd = None
+                source = src_entry
+
+            # check if the sources can be located
             path = self.obtain_file(source)
             if path:
                 self.log.debug('File %s found for source %s' % (path, source))
-                self.src.append({'name':source, 'path':path})
+                self.src.append({'name': source, 'path': path, 'cmd': cmd})
             else:
                 self.log.error('No file found for source %s' % source)
 
@@ -170,7 +185,7 @@ class EasyBlock(object):
         for patchFile in list_of_patches:
 
             ## check if the patches can be located
-            copy = False
+            copy_file = False
             suff = None
             level = None
             if type(patchFile) in [list, tuple]:
@@ -183,7 +198,7 @@ class EasyBlock(object):
                 elif type(patchFile[1]) == str:
                     # non-patch files are assumed to be files to copy
                     if not patchFile[0].endswith('.patch'):
-                        copy = True
+                        copy_file = True
                     suff = patchFile[1]
                 else:
                     self.log.error("Wrong patch specification '%s', only int and string are supported as second element!" % patchFile)
@@ -195,7 +210,7 @@ class EasyBlock(object):
                 self.log.debug('File %s found for patch %s' % (path, patchFile))
                 tmppatch = {'name':pf, 'path':path}
                 if suff:
-                    if copy:
+                    if copy_file:
                         tmppatch['copy'] = suff
                     else:
                         tmppatch['sourcepath'] = suff
@@ -341,20 +356,24 @@ class EasyBlock(object):
             # try and find file in various locations
             foundfile = None
             failedpaths = []
-            for srcpath in srcpaths:
+
+            # always consider robot + easyconfigs install paths as a fall back (e.g. for patch files, test cases, ...)
+            common_filepaths = []
+            if not self.robot_path is None:
+                common_filepaths.append(self.robot_path)
+            common_filepaths.extend(get_paths_for(self.log, "easyconfigs", robot_path=self.robot_path))
+
+            for path in common_filepaths + srcpaths:
                 # create list of candidate filepaths
-                namepath = os.path.join(srcpath, self.name)
-                fst_letter_path_low = os.path.join(srcpath, self.name.lower()[0])
+                namepath = os.path.join(path, self.name)
+                letterpath = os.path.join(path, self.name.lower()[0], self.name)
 
                 # most likely paths
-                candidate_filepaths = [os.path.join(fst_letter_path_low, self.name), # easyblocks-style subdir
-                                       namepath, # subdir with software name
-                                       srcpath, # directly in sources directory
-                                       ]
-
-                # also consider easyconfigs paths as a fall back (e.g. for patch files, test cases, ...)
-                for path in get_paths_for(self.log, "easyconfigs"):
-                    candidate_filepaths.append(os.path.join(path, self.name.lower()[0], self.name))
+                candidate_filepaths = [
+                                       letterpath,  # easyblocks-style subdir
+                                       namepath,  # subdir with software name
+                                       path,  # directly in directory
+                                      ]
 
                 # see if file can be found at that location
                 for cfp in candidate_filepaths:
@@ -374,7 +393,7 @@ class EasyBlock(object):
                     for fp in fullpaths:
                         if os.path.isfile(fp):
                             self.log.info("Found file %s at %s" % (filename, fp))
-                            foundfile = fp
+                            foundfile = os.path.abspath(fp)
                             break # no need to try further
                         else:
                             failedpaths.append(fp)
@@ -407,7 +426,10 @@ class EasyBlock(object):
                         targetpath = os.path.join(targetdir, filename)
 
                     if type(url) == str:
-                        fullurl = "%s/%s" % (url, filename)
+                        if url[-1] in ['=', '/']:
+                            fullurl = "%s%s" % (url, filename)
+                        else:
+                            fullurl = "%s/%s" % (url, filename)
                     elif type(url) == tuple:
                         ## URLs that require a suffix, e.g., SourceForge download links
                         ## e.g. http://sourceforge.net/projects/math-atlas/files/Stable/3.8.4/atlas3.8.4.tar.bz2/download
@@ -566,7 +588,7 @@ class EasyBlock(object):
         except OSError, err:
             self.log.exception("Can't create directory %s: %s" % (dirName, err))
 
-    # 
+    #
     # MODULE UTILITY FUNCTIONS
     #
 
@@ -578,20 +600,11 @@ class EasyBlock(object):
 
         WARNING: you cannot unload using $EBDEVELNAME (for now: use module unload `basename $EBDEVELNAME`)
         """
-        # first try loading the fake module (might have happened during sanity check, doesn't matter anyway
-        # make fake module
-        mod_path = [self.make_module_step(True)]
 
-        # load the module
-        mod_path.extend(Modules().modulePath)
-        m = Modules(mod_path)
-        self.log.debug("created module instance")
-        m.add_module([[self.name, self.get_installversion()]])
-        try:
-            m.load()
-        except EasyBuildError, err:
-            self.log.debug("Loading module failed: %s" % err)
-            self.log.debug("loaded modules: %s" % Modules().loaded_modules())
+        self.log.info("Making devel module...")
+
+        # load fake module
+        fake_mod_data = self.load_fake_module(purge=True)
 
         mod_gen = ModuleGenerator(self)
         header = "#%Module\n"
@@ -631,6 +644,9 @@ class EasyBlock(object):
         devel_module.write(env_txt)
         devel_module.close()
 
+        # cleanup: unload fake module, remove fake module dir
+        self.clean_up_fake_module(fake_mod_data)
+
     def make_module_dep(self):
         """
         Make the dependencies for the module file.
@@ -690,7 +706,14 @@ class EasyBlock(object):
         """
         Sets optional variables for extensions.
         """
-        return self.module_extra_extensions
+        txt = self.module_extra_extensions
+
+        # set environment variable that specifies list of extensions
+        if self.exts:
+            exts_list = ','.join(['%s-%s' % (ext['name'], ext.get('version', '')) for ext in self.exts])
+            txt += self.moduleGenerator.set_environment('EBEXTSLIST%s' % self.name.upper(), exts_list)
+
+        return txt
 
     def make_module_req(self):
         """
@@ -717,27 +740,70 @@ class EasyBlock(object):
             'PKG_CONFIG_PATH' : ['lib/pkgconfig', 'share/pkgconfig'],
         }
 
-    def load_fake_module(self):
+    def load_module(self):
+        """
+        Load module for this software package/version.
+        """
+        m = Modules()
+        m.add_module([[self.name, self.get_installversion()]])
+        m.load()
+
+    def load_fake_module(self, purge=False):
         """
         Create and load fake module.
         """
 
+        # take a copy of the environment before loading the fake module
+        orig_env = copy.deepcopy(os.environ)
+
         # make fake module
         fake_mod_path = self.make_module_step(True)
 
-        # load the module
+        # create Modules instance
         mod_paths = [fake_mod_path]
-        mod_paths.extend(Modules().modulePath)
+        mod_paths.extend(self.orig_modulepath.split(':'))
         m = Modules(mod_paths)
-        self.log.debug("created module instance")
+        self.log.debug("mod_paths: %s" % mod_paths)
+
+        # purge loaded modules if desired
+        if purge:
+            m.purge()
+
+        # make sure MODULEPATH is set correctly after purging
+        m.check_module_path()
+
+        # load the module
         m.add_module([[self.name, self.get_installversion()]])
         m.load()
 
-        # clean up
-        try:
-            rmtree2(os.path.dirname(fake_mod_path))
-        except OSError, err:
-            self.log.error("Failed to clean up fake module dir: %s" % err)
+        return (fake_mod_path, orig_env)
+
+    def clean_up_fake_module(self, fake_mod_data):
+        """
+        Clean up fake module.
+        """
+
+        fake_mod_path, orig_env = fake_mod_data
+
+        # unload module and remove temporary module directory
+        if fake_mod_path:
+            try:
+                mod_paths = [fake_mod_path]
+                mod_paths.extend(Modules().modulePath)
+                m = Modules(mod_paths)
+                m.add_module([[self.name, self.get_installversion()]])
+                m.unload()
+                rmtree2(os.path.dirname(fake_mod_path))
+            except OSError, err:
+                self.log.error("Failed to clean up fake module dir: %s" % err)
+
+        # restore env vars that were set before loading the fake module,
+        # but were unset after unloading it because the module also sets them
+        curr_env = copy.deepcopy(os.environ)
+        for (var, val) in orig_env.items():
+            if not var in curr_env:
+                os.environ[var] = val
+                self.log.debug("Restored value %s for %s that was unset by unloading fake module." % (val, var))
 
     #
     # EXTENSIONS UTILITY FUNCTIONS
@@ -755,17 +821,20 @@ class EasyBlock(object):
         - use this to detect existing extensions and to remove them from self.exts
         - based on initial R version
         """
-        if not self.cfg['exts_filter'] or len(self.cfg['exts_filter']) == 0:
+        exts_filter = self.cfg['exts_filter']
+        if not exts_filter or len(exts_filter) == 0:
             self.log.error("Skipping of extensions, but no exts_filter set in easyconfig")
-        elif len(self.cfg['exts_filter']) == 1:
+        elif isinstance(exts_filter, basestring) or len(exts_filter) != 2:
             self.log.error('exts_filter should be a list or tuple of ("command","input")')
-        cmdtmpl = self.cfg['exts_filter'][0]
-        cmdinputtmpl = self.cfg['exts_filter'][1]
+        cmdtmpl = exts_filter[0]
+        cmdinputtmpl = exts_filter[1]
+        if not self.exts:
+            self.exts = []
 
         res = []
         for ext in self.exts:
             name = ext['name']
-            if 'modulename' in ext['options']:
+            if 'options' in ext and 'modulename' in ext['options']:
                 modname = ext['options']['modulename']
             else:
                 modname = name
@@ -777,12 +846,13 @@ class EasyBlock(object):
             cmd = cmdtmpl % tmpldict
             if cmdinputtmpl:
                 stdin = cmdinputtmpl % tmpldict
-                (cmdStdouterr, ec) = run_cmd(cmd, log_all=False, log_ok=False, simple=False, inp=stdin, regexp=False)
+                (cmdstdouterr, ec) = run_cmd(cmd, log_all=False, log_ok=False, simple=False, inp=stdin, regexp=False)
             else:
-                (cmdStdouterr, ec) = run_cmd(cmd, log_all=False, log_ok=False, simple=False, regexp=False)
+                (cmdstdouterr, ec) = run_cmd(cmd, log_all=False, log_ok=False, simple=False, regexp=False)
+            self.log.info("exts_filter result %s %s", cmdstdouterr, ec)
             if ec:
                 self.log.info("Not skipping %s" % name)
-                self.log.debug("exit code: %s, stdout/err: %s" % (ec, cmdStdouterr))
+                self.log.debug("exit code: %s, stdout/err: %s" % (ec, cmdstdouterr))
                 res.append(ext)
             else:
                 self.log.info("Skipping %s" % name)
@@ -793,6 +863,7 @@ class EasyBlock(object):
     #
 
     def det_installsize(self):
+        """Determine size of installation."""
         installsize = 0
         try:
             # change to home dir, to avoid that cwd no longer exists
@@ -810,6 +881,7 @@ class EasyBlock(object):
         return installsize
 
     def get_installversion(self):
+        """Get full install version (including toolchain and version suffix)."""
         return self.cfg.get_installversion()
 
     def guess_start_dir(self):
@@ -1021,7 +1093,8 @@ class EasyBlock(object):
         """
         for tmp in self.src:
             self.log.info("Unpacking source %s" % tmp['name'])
-            srcdir = extract_file(tmp['path'], self.builddir, extra_options=self.cfg['unpack_options'])
+            srcdir = extract_file(tmp['path'], self.builddir, cmd=tmp['cmd'],
+                                  extra_options=self.cfg['unpack_options'])
             if srcdir:
                 self.src[self.src.index(tmp)]['finalpath'] = srcdir
             else:
@@ -1104,30 +1177,8 @@ class EasyBlock(object):
             self.log.debug("No extensions in exts_list")
             return
 
-        if not self.skip:
-            modpath = self.make_module_step(fake=True)
-
-        # adjust MODULEPATH and load module
-        if self.skip:
-            m = Modules()
-        else:
-            self.log.debug("Adding %s to MODULEPATH" % modpath)
-            m = Modules([modpath] + os.environ['MODULEPATH'].split(':'))
-
-
-        if m.exists(self.name, self.get_installversion()):
-            m.add_module([[self.name, self.get_installversion()]])
-            m.load()
-        else:
-            self.log.error("module %s version %s doesn't exist" % (self.name, self.get_installversion()))
-
-        if not self.skip:
-            try:
-                fakemoddir = os.path.dirname(modpath)
-                self.log.debug("Cleaning up fake module dir %s..." % fakemoddir)
-                rmtree2(fakemoddir)
-            except OSError, err:
-                self.log.error("Failed to clean up fake module dir: %s" % err)
+        # load fake module
+        fake_mod_data = self.load_fake_module(purge=True)
 
         self.prepare_for_extensions()
 
@@ -1139,43 +1190,112 @@ class EasyBlock(object):
         # actually install extensions
         self.log.debug("Installing extensions")
         exts_defaultclass = self.cfg['exts_defaultclass']
+        exts_classmap = self.cfg['exts_classmap']
+
+        # we really need a default class
         if not exts_defaultclass:
+            self.clean_up_fake_module(fake_mod_data)
             self.log.error("ERROR: No default extension class set for %s" % self.name)
 
-        # exts_defaultclass should be a list or a tuple, but certaintly not a string
-        assert not isinstance(exts_defaultclass, basestring)
-        allclassmodule = exts_defaultclass[0]
-        defaultClass = exts_defaultclass[1]
+        # obtain name and module path for default extention class
+        legacy = False
+        if hasattr(exts_defaultclass, '__iter__'):
+            # LEGACY: module path is explicitely specified
+            self.log.warning("LEGACY: using specified module path for default class (will be deprecated soon)")
+            default_class_modpath = exts_defaultclass[0]
+            default_class = exts_defaultclass[1]
+            derived_mod_path = get_module_path(default_class, generic=True)
+            if not default_class_modpath == derived_mod_path:
+                msg = "Specified module path for default class %s " % default_class_modpath
+                msg += "doesn't match derived path %s" % derived_mod_path
+                self.log.warning(msg)
+            legacy = True
+
+        elif isinstance(exts_defaultclass, basestring):
+            # proper way: derive module path from specified class name
+            default_class = exts_defaultclass
+            default_class_modpath = get_module_path(default_class, generic=True)
+
+        else:
+            self.log.error("Improper default extension class specification, should be list/tuple or string.")
+
+        # get class instances for all extensions
         for ext in self.exts:
-            name = encode_class_name(ext['name']) # Use the same encoding as get_class
-            self.log.debug("Starting extension %s" % name)
+            self.log.debug("Starting extension %s" % ext['name'])
 
+            # always go back to build dir to avoid running stuff from a dir that no longer exists
+            os.chdir(self.builddir)
+
+            inst = None
+
+            # try instantiating extension-specific class
+            class_name = encode_class_name(ext['name'])  # use the same encoding as get_class
+            mod_path = get_module_path(ext['name'])
             try:
-                cls = get_class_for(allclassmodule, name)
-                p = cls(self, ext)
-                self.log.debug("Installing extension %s through class %s" % (ext['name'], name))
+                cls = get_class_for(mod_path, class_name)
+                inst = cls(self, ext)
             except (ImportError, NameError), err:
-                self.log.debug("Couldn't load class %s for extension %s:\n%s" % (name, ext['name'], err))
-                if defaultClass:
-                    self.log.info("No class found for %s, using default %s instead." % (ext['name'], defaultClass))
-                    try:
-                        cls = get_class_for(allclassmodule, defaultClass)
-                        p = cls(self, ext)
-                        self.log.debug("Installing extension %s through default class %s" % (ext['name'], defaultClass))
-                    except (ImportError, NameError), errbis:
-                        self.log.error("Failed to use both class %s and default %s for extension %s, giving up:\n%s\n%s" % (name, defaultClass, ext['name'], err, errbis))
-                else:
-                    self.log.error("Failed to use both class %s and no default class for extension %s, giving up:\n%s" % (name, ext['name'], err))
+                self.log.debug("Failed to use class %s from %s for extension %s: %s" % (class_name,
+                                                                                        mod_path,
+                                                                                        ext['name'],
+                                                                                        err))
 
-            ## real work
-            p.prerun()
-            txt = p.run()
+            # LEGACY: try and use default module path for getting extension class instance
+            if inst is None and legacy:
+                try:
+                    msg = "Failed to use derived module path for %s, " % class_name
+                    msg += "considering specified module path as (legacy) fallback."
+                    self.log.debug(msg)
+                    mod_path = default_class_modpath
+                    cls = get_class_for(mod_path, class_name)
+                    inst = cls(self, ext)
+                except (ImportError, NameError), err:
+                    self.log.debug("Failed to use class %s from %s for extension %s: %s" % (class_name,
+                                                                                            mod_path,
+                                                                                            ext['name'],
+                                                                                            err))
+
+            # alternative attempt: use class specified in class map (if any)
+            if inst is None and ext['name'] in exts_classmap:
+
+                class_name = exts_classmap[ext['name']]
+                mod_path = get_module_path(class_name)
+                try:
+                    cls = get_class_for(mod_path, class_name)
+                    inst = cls(self, ext)
+                except (ImportError, NameError), err:
+                    self.log.error("Failed to load specified class %s for extension %s: %s" % (class_name,
+                                                                                               ext['name'],
+                                                                                               err))
+
+            # fallback attempt: use default class
+            if not inst is None:
+                self.log.debug("Installing extension %s with class %s (from %s)" % (ext['name'], class_name, mod_path))
+            else:
+                try:
+                    cls = get_class_for(default_class_modpath, default_class)
+                    inst = cls(self, ext)
+                    self.log.debug("Installing extension %s with default class %s" % (ext['name'], default_class))
+                except (ImportError, NameError), errbis:
+                    msg = "Also failed to use default class %s from %s for extension %s: %s" % (default_class,
+                                                                                                default_class_modpath,
+                                                                                                ext['name'],
+                                                                                                err)
+                    msg += ", giving up:\n%s\n%s" % (err, errbis)
+                    self.log.error(msg)
+
+            # real work
+            inst.prerun()
+            txt = inst.run()
             if txt:
                 self.module_extra_extensions += txt
-            p.postrun()
-            # Append so we can make us of it later (in sanity_check)
-            self.ext_instances.append(p)
+            inst.postrun()
 
+            # append so we can make us of it later (in sanity_check_step)
+            self.ext_instances.append(inst)
+
+        # cleanup (unload fake module, remove fake module dir)
+        self.clean_up_fake_module(fake_mod_data)
 
     def package_step(self):
         """Package software (e.g. into an RPM)."""
@@ -1264,10 +1384,13 @@ class EasyBlock(object):
                 else:
                     self.log.debug("Sanity check: found non-empty directory %s in %s" % (d, self.installdir))
 
+        fake_mod_data = None
         try:
-            self.load_fake_module()
+            # unload all loaded modules before loading fake module
+            # this ensures that loading of dependencies is tested, and avoids conflicts with build dependencies
+            fake_mod_data = self.load_fake_module(purge=True)
         except EasyBuildError, err:
-            self.log.debug("Loading fake module failed: %s" % err)
+            self.log.info("Loading fake module failed: %s" % err)
             self.sanityCheckOK = False
 
         # chdir to installdir (better environment for running tests)
@@ -1314,6 +1437,10 @@ class EasyBlock(object):
             self.log.info("Sanity check for extensions %s failed!" % failed_exts)
             self.sanityCheckOK = False
 
+        # cleanup
+        if fake_mod_data:
+            self.clean_up_fake_module(fake_mod_data)
+
         # pass or fail
         if not self.sanityCheckOK:
             self.log.error("Sanity check failed!")
@@ -1329,6 +1456,10 @@ class EasyBlock(object):
         """
         if not self.build_in_installdir:
             try:
+                os.chdir(build_path())  # make sure we're out of the dir we're removing
+
+                self.log.info("Cleaning up builddir %s (in %s)" % (self.builddir, os.getcwd()))
+
                 rmtree2(self.builddir)
                 base = os.path.dirname(self.builddir)
 
@@ -1338,7 +1469,6 @@ class EasyBlock(object):
                     os.rmdir(base)
                     base = os.path.dirname(base)
 
-                self.log.info("Cleaning up builddir %s" % (self.builddir))
             except OSError, err:
                 self.log.exception("Cleaning up builddir %s failed: %s" % (self.builddir, err))
 
@@ -1439,7 +1569,10 @@ class EasyBlock(object):
                   ]
 
         if run_test_cases:
-            steps.append(('testcases', 'running test cases', [lambda x: x.test_cases_step()], False))
+            steps.append(('testcases', 'running test cases', [
+                                                              lambda x: x.load_module(),
+                                                              lambda x: x.test_cases_step(),
+                                                             ], False))
 
         return steps
 
