@@ -32,9 +32,13 @@
 Generic EasyBuild support for software extensions (e.g. Python packages).
 The Extension class should serve as a base class for all extensions.
 """
+import copy
+import os
 
+from easybuild.tools.config import build_path
 from easybuild.tools.filetools import run_cmd
 
+
 class Extension(object):
     """
     Support for installing extensions.
@@ -45,17 +49,31 @@ class Extension(object):
         """
         self.master = mself
         self.log = self.master.log
-        self.cfg = self.master.cfg
-        self.ext = ext
+        self.cfg = self.master.cfg.copy()
+        self.ext = copy.deepcopy(ext)
 
         if not 'name' in self.ext:
-            self.log.error("")
+            self.log.error("'name' is missing in supplied class instance 'ext'.")
 
-        self.name = self.ext.get('name', None)
-        self.version = self.ext.get('version', None)
         self.src = self.ext.get('src', None)
         self.patches = self.ext.get('patches', None)
-        self.options = self.ext.get('options', {})
+        self.options = copy.deepcopy(self.ext.get('options', {}))
+
+        self.toolchain.prepare(self.cfg['onlytcmod'])
+
+    @property
+    def name(self):
+        """
+        Shortcut the get the extension name.
+        """
+        return self.ext.get('name', None)
+
+    @property
+    def version(self):
+        """
+        Shortcut the get the extension version.
+        """
+        return self.ext.get('version', None)
 
     def prerun(self):
         """
@@ -84,11 +102,17 @@ class Extension(object):
 
     def sanity_check_step(self):
         """
-        sanity check to run after installing
+        Sanity check to run after installing extension
         """
+
         try:
-            cmd, inp = self.master.cfg['exts_filter']
-        except:
+            os.chdir(build_path())
+        except OSError, err:
+            self.log.error("Failed to change directory: %s" % err)
+
+        if not self.cfg['exts_filter'] is None:
+            cmd, inp = self.cfg['exts_filter']
+        else:
             self.log.debug("no exts_filter setting found, skipping sanitycheck")
             return
 
@@ -97,21 +121,27 @@ class Extension(object):
         else:
             modname = self.name
 
-        template = {
-                    'name': modname,
-                    'version': self.version,
-                    'src': self.src
-                   }
-        cmd = cmd % template
-
-        if inp:
-            stdin = inp % template
-            # set log_ok to False so we can catch the error instead of run_cmd
-            (output, ec) = run_cmd(cmd, log_ok=False, simple=False, inp=stdin, regexp=False)
-        else:
-            (output, ec) = run_cmd(cmd, log_ok=False, simple=False, regexp=False)
-        if ec:
-            self.log.warn("Extension: %s failed to install! (output: %s)" % (self.name, output))
-            return False
-        else:
+        if modname == False:
+            # allow skipping of sanity check by setting module name to False
             return True
+
+        else:
+            template = {
+                        'name': modname,
+                        'version': self.version,
+                        'src': self.src
+                       }
+            cmd = cmd % template
+
+            if inp:
+                stdin = inp % template
+                # set log_ok to False so we can catch the error instead of run_cmd
+                (output, ec) = run_cmd(cmd, log_ok=False, simple=False, inp=stdin, regexp=False)
+            else:
+                (output, ec) = run_cmd(cmd, log_ok=False, simple=False, regexp=False)
+
+            if ec:
+                self.log.warn("Extension: %s failed to install! (output: %s)" % (self.name, output))
+                return False
+            else:
+                return True
