@@ -77,16 +77,22 @@ import easybuild.tools.config as config
 import easybuild.tools.filetools as filetools
 import easybuild.tools.parallelbuild as parbuild
 from easybuild.framework.easyblock import EasyBlock, get_class
-from easybuild.framework.easyconfig import EasyConfig
+from easybuild.framework.easyconfig import EasyConfig, get_paths_for
+from easybuild.tools import systemtools
 from easybuild.tools.build_log import EasyBuildError, init_logger
 from easybuild.tools.build_log import remove_log_handler, print_msg
-from easybuild.tools.config import get_repository
-from easybuild.tools.filetools import modify_env
+from easybuild.tools.config import get_repository, module_classes
+from easybuild.tools.filetools import modify_env, run_cmd
 from easybuild.tools.modules import Modules, search_module
 from easybuild.tools.modules import curr_module_paths, mk_module_path
+from easybuild.tools.toolchain.utilities import search_toolchain
 from easybuild.tools.ordereddict import OrderedDict
-from easybuild.tools.version import VERBOSE_VERSION
-from easybuild.tools import systemtools
+from easybuild.tools.version import VERBOSE_VERSION as FRAMEWORK_VERSION
+EASYBLOCKS_VERSION = 'UNKNOWN'
+try:
+    from easybuild.easyblocks import VERBOSE_VERSION as EASYBLOCKS_VERSION
+except:
+    pass
 
 
 # applications use their own logger, we need to tell them to debug or not
@@ -131,7 +137,7 @@ def add_cmdline_options(parser):
                              "(default: easybuild-easyconfigs install path)")
     basic_options.add_option("-s", "--stop", type="choice", choices=all_stops,
                         help="stop the installation after certain step (valid: %s)" % ', '.join(all_stops))
-    strictness_options = ['ignore', 'warn', 'error']
+    strictness_options = [filetools.IGNORE, filetools.WARN, filetools.ERROR]
     basic_options.add_option("--strict", type="choice", choices=strictness_options, help="set strictness " + \
                                "level (possible levels: %s)" % ', '.join(strictness_options))
 
@@ -200,6 +206,7 @@ def add_cmdline_options(parser):
     # see https://github.com/hpcugent/VSC-tools/issues/8
     informative_options.add_option("--list-easyblocks", type="choice", choices=["simple", "detailed"], default=None,
                                    help="show list of available easyblocks ('simple' or 'detailed')")
+    informative_options.add_option("--list-toolchains", action="store_true", help="show list of known toolchains")
     informative_options.add_option("--search", metavar="STR", help="search for module-files in the robot-directory")
     informative_options.add_option("-v", "--version", action="store_true", help="show version")
     informative_options.add_option("--dep-graph", metavar="depgraph.<ext>", help="create dependency graph")
@@ -259,44 +266,40 @@ def main(options, orig_paths, log, logfile, hn, parser):
     - build software
     """
 
+    # set strictness of filetools module
+    if options.strict:
+        filetools.strictness = options.strict
+
     # disallow running EasyBuild as root
     if os.getuid() == 0:
-        sys.stderr.write("ERROR: You seem to be running EasyBuild with root priveleges.\n" \
+        sys.stderr.write("ERROR: You seem to be running EasyBuild with root privileges.\n" \
                         "That's not wise, so let's end this here.\n" \
                         "Exiting.\n")
         sys.exit(1)
 
     # show version
     if options.version:
-        print_msg("This is EasyBuild %s" % VERBOSE_VERSION, log)
+        top_version = max(FRAMEWORK_VERSION, EASYBLOCKS_VERSION)
+        print_msg("This is EasyBuild %s (framework: %s, easyblocks: %s)" % (top_version,
+                                                                            FRAMEWORK_VERSION,
+                                                                            EASYBLOCKS_VERSION), log)
 
     # determine easybuild-easyconfigs package install path
-    # we may need for the robot (default path), or for finding easyconfig files
-    easyconfigs_pkg_base_path = os.path.join('easybuild', 'easyconfigs')
+    easyconfigs_paths = get_paths_for(log, "easyconfigs", robot_path=options.robot)
     easyconfigs_pkg_full_path = None
-    for syspath in sys.path:
-        tmppath = os.path.join(syspath, easyconfigs_pkg_base_path)
-        if os.path.basename(syspath).startswith('easybuild_easyconfigs') and os.path.exists(tmppath):
-            easyconfigs_pkg_full_path = tmppath
-            log.info("Found path for easyconfigs in Python search path: %s" % easyconfigs_pkg_full_path)
-            break
 
-    if not easyconfigs_pkg_full_path:
-        log.info("Failed to determine easyconfigs path for easybuild-easyconfigs package.")
+    if easyconfigs_paths:
+        easyconfigs_pkg_full_path = easyconfigs_paths[0]
+    else:
+        log.info("Failed to determine install path for easybuild-easyconfigs package.")
 
     if options.robot and type(options.robot) == bool:
-        if easyconfigs_pkg_full_path:
+        if not easyconfigs_pkg_full_path is None:
             options.robot = easyconfigs_pkg_full_path
-            log.info("Using default robot path: %s" % options.robot)
+            log.info("Using default robot path (easyconfigs install dir): %s" % options.robot)
         else:
             log.error("No robot path specified, and unable to determine easybuild-easyconfigs install path.")
 
-    # initialize configuration
-    # - check environment variable EASYBUILDCONFIG
-    # - then, check command line option
-    # - last, use default config file easybuild_config.py in main.py directory
-    config_file = options.config
-
     configOptions = {}
     if options.pretend:
         configOptions['install_path'] = os.path.join(os.environ['HOME'], 'easybuildinstall')
@@ -306,13 +309,23 @@ def main(options, orig_paths, log, logfile, hn, parser):
     else:
         blocks = None
 
+    # initialize configuration
+    # - check command line option -C/--config
+    # - then, check environment variable EASYBUILDCONFIG
+    # - next, check for an EasyBuild config in $HOME/.easybuild/config.py
+    # - last, use default config file easybuild_config.py in main.py directory
+    config_file = options.config
     if not config_file:
         log.debug("No config file specified on command line, trying other options.")
 
         config_env_var = config.environmentVariables['config_file']
+        home_config_file = os.path.join(os.getenv('HOME'), ".easybuild", "config.py")
         if os.getenv(config_env_var):
             log.debug("Environment variable %s, so using that as config file." % config_env_var)
             config_file = os.getenv(config_env_var)
+        elif os.path.exists(home_config_file):
+            config_file = home_config_file
+            log.debug("Found EasyBuild configuration file at %s." % config_file)
         else:
             appPath = os.path.dirname(os.path.realpath(sys.argv[0]))
             config_file = os.path.join(appPath, "easybuild_config.py")
@@ -324,10 +337,14 @@ def main(options, orig_paths, log, logfile, hn, parser):
     if options.avail_easyconfig_params:
         print_avail_params(options.easyblock, log)
 
-    # dump available classes
+    # dump available easyblocks
     if options.list_easyblocks:
         list_easyblocks(detailed=options.list_easyblocks=="detailed")
 
+    # dump known toolchains
+    if options.list_toolchains:
+        list_toolchains()
+
     # search for modules
     if options.search:
         if not options.robot:
@@ -343,7 +360,7 @@ def main(options, orig_paths, log, logfile, hn, parser):
         if software_build_specs.has_key('name'):
             paths = [obtain_path(software_build_specs, options.robot, log, try_to_generate)]
         elif not any([options.aggregate_regtest, options.avail_easyconfig_params, options.list_easyblocks,
-                     options.search, options.regtest, options.version]):
+                      options.list_toolchains, options.search, options.regtest, options.version]):
             error("Please provide one or multiple easyconfig files, or use software build " \
                   "options to make EasyBuild search for easyconfigs", optparser=parser)
 
@@ -382,15 +399,12 @@ def main(options, orig_paths, log, logfile, hn, parser):
             log.info("Regression test failed (partially)!")
             sys.exit(31)  # exit -> 3x1t -> 31
 
-    if options.avail_easyconfig_params or options.list_easyblocks or options.search or options.version or options.regtest:
+    if any([options.avail_easyconfig_params, options.list_easyblocks, options.list_toolchains, options.search,
+             options.version, options.regtest]):
         if logfile:
             os.remove(logfile)
         sys.exit(0)
 
-    # set strictness of filetools module
-    if options.strict:
-        filetools.strictness = options.strict
-
     # building a dependency graph implies force, so that all dependencies are retained
     # and also skips validation of easyconfigs (e.g. checking os dependencies)
     validate_easyconfigs = True
@@ -492,9 +506,10 @@ def main(options, orig_paths, log, logfile, hn, parser):
 
         opts = ' '.join(result_opts)
 
-        command = "cd %s && eb %%(spec)s %s" % (curdir, opts)
+        command = "unset TMPDIR && cd %s && eb %%(spec)s %s" % (curdir, opts)
         log.debug("Command template for jobs: %s" % command)
-        jobs = parbuild.build_easyconfigs_in_parallel(command, orderedSpecs, "easybuild-build", log)
+        jobs = parbuild.build_easyconfigs_in_parallel(command, orderedSpecs, "easybuild-build", log,
+                                                      robot_path=options.robot)
         print "List of submitted jobs:"
         for job in jobs:
             print "%s: %s" % (job.name, job.jobid)
@@ -581,7 +596,7 @@ def process_easyconfig(path, log, onlyBlocks=None, regtest_online=False, validat
         # create easyconfig
         try:
             all_stops = [x[0] for x in EasyBlock.get_steps()]
-            ec = EasyConfig(spec, validate=validate, valid_stops=all_stops)
+            ec = EasyConfig(spec, validate=validate, valid_module_classes=module_classes(), valid_stops=all_stops)
         except EasyBuildError, err:
             msg = "Failed to process easyconfig %s:\n%s" % (spec, err.msg)
             log.exception(msg)
@@ -929,7 +944,8 @@ def get_build_stats(app, starttime):
 
     buildtime = round(time.time() - starttime, 2)
     buildstats = OrderedDict([
-                              ('easybuild_version', str(VERBOSE_VERSION)),
+                              ('easybuild-framework_version', str(FRAMEWORK_VERSION)),
+                              ('easybuild-easyblocks_version', str(EASYBLOCKS_VERSION)),
                               ('host', os.uname()[1]),
                               ('platform' , platform.platform()),
                               ('cpu_model', systemtools.get_cpu_model()),
@@ -970,7 +986,7 @@ def build_and_install_software(module, options, log, origEnviron, exitOnFailure=
     name = module['module'][0]
     try:
         app_class = get_class(easyblock, log, name=name)
-        app = app_class(spec, debug=options.debug)
+        app = app_class(spec, debug=options.debug, robot_path=options.robot)
         log.info("Obtained application instance of for %s (easyblock: %s)" % (name, easyblock))
     except EasyBuildError, err:
         error("Failed to get application instance for %s (easyblock: %s): %s" % (name, easyblock, err.msg))
@@ -1183,10 +1199,14 @@ def write_to_xml(succes, failed, filename):
         return el
 
     properties = root.createElement("properties")
-    version = root.createElement("property")
-    version.setAttribute("name", "easybuild-version")
-    version.setAttribute("value", str(VERBOSE_VERSION))
-    properties.appendChild(version)
+    framework_version = root.createElement("property")
+    framework_version.setAttribute("name", "easybuild-framework-version")
+    framework_version.setAttribute("value", str(FRAMEWORK_VERSION))
+    properties.appendChild(framework_version)
+    easyblocks_version = root.createElement("property")
+    easyblocks_version.setAttribute("name", "easybuild-easyblocks-version")
+    easyblocks_version.setAttribute("value", str(EASYBLOCKS_VERSION))
+    properties.appendChild(easyblocks_version)
 
     time = root.createElement("property")
     time.setAttribute("name", "timestamp")
@@ -1225,7 +1245,7 @@ def build_easyconfigs(easyconfigs, output_dir, test_results, options, log):
             try:
                 if step == 'initialization':
                     log.info("Running %s step" % step)
-                    return parbuild.get_instance(obj, log)
+                    return parbuild.get_instance(obj, log, robot_path=options.robot)
                 else:
                     apploginfo(obj, "Running %s step" % step)
                     method(obj)
@@ -1318,10 +1338,14 @@ def aggregate_xml_in_dirs(base_dir, output_filename):
     root = dom.createDocument(None, "testsuite", None)
     root.documentElement.setAttribute("name", base_dir)
     properties = root.createElement("properties")
-    version = root.createElement("property")
-    version.setAttribute("name", "easybuild-version")
-    version.setAttribute("value", str(VERBOSE_VERSION))
-    properties.appendChild(version)
+    framework_version = root.createElement("property")
+    framework_version.setAttribute("name", "easybuild-framework-version")
+    framework_version.setAttribute("value", str(FRAMEWORK_VERSION))
+    properties.appendChild(framework_version)
+    easyblocks_version = root.createElement("property")
+    easyblocks_version.setAttribute("name", "easybuild-easyblocks-version")
+    easyblocks_version.setAttribute("value", str(EASYBLOCKS_VERSION))
+    properties.appendChild(easyblocks_version)
 
     time_el = root.createElement("property")
     time_el.setAttribute("name", "timestamp")
@@ -1400,7 +1424,7 @@ def regtest(options, log, easyconfig_paths):
         try:
             easyconfigs.extend(process_easyconfig(ecfile, log, None))
         except EasyBuildError, err:
-            test_results.append((ecfile, 'easyconfig file error', err))
+            test_results.append((ecfile, 'parsing_easyconfigs', 'easyconfig file error: %s' % err, log))
 
     if options.sequential:
         return build_easyconfigs(easyconfigs, output_dir, test_results, options, log)
@@ -1408,11 +1432,11 @@ def regtest(options, log, easyconfig_paths):
         resolved = resolve_dependencies(easyconfigs, options.robot, log)
 
         cmd = "eb %(spec)s --regtest --sequential -ld"
-        command = "cd %s && %s; " % (cur_dir, cmd)
+        command = "unset TMPDIR && cd %s && %s; " % (cur_dir, cmd)
         # retry twice in case of failure, to avoid fluke errors
         command += "if [ $? -ne 0 ]; then %(cmd)s && %(cmd)s; fi" % {'cmd': cmd}
 
-        jobs = parbuild.build_easyconfigs_in_parallel(command, resolved, output_dir, log)
+        jobs = parbuild.build_easyconfigs_in_parallel(command, resolved, output_dir, log, robot_path=options.robot)
 
         print "List of submitted jobs:"
         for job in jobs:
@@ -1489,6 +1513,8 @@ def list_easyblocks(detailed=False):
             print ""
 
 def print_tree(classes, classNames, detailed, depth=0):
+    """Print list of classes as a tree."""
+
     for className in classNames:
         classInfo = classes[className]
         if detailed:
@@ -1498,6 +1524,22 @@ def print_tree(classes, classNames, detailed, depth=0):
         if 'children' in classInfo:
             print_tree(classes, classInfo['children'], detailed, depth + 1)
 
+def list_toolchains():
+    """Show list of known toolchains."""
+
+    _, all_tcs = search_toolchain('')
+    all_tcs_names = [x.NAME for x in all_tcs]
+    tclist = sorted(zip(all_tcs_names, all_tcs))
+
+    print "List of known toolchains:"
+
+    for (tcname, tcc) in tclist:
+
+        tc = tcc(version='1.2.3')  # version doesn't matter here, but something needs to be there
+        tc_elems = set([y for x in dir(tc) if x.endswith('_MODULE_NAME') for y in eval("tc.%s" % x)])
+
+        print "\t%s: %s" % (tcname, ', '.join(sorted(tc_elems)))
+
 # FIXME: remove when Python version on which we rely provides any by itself
 def any(ls):
     """Reimplementation of 'any' function, which is not available in Python 2.4 yet."""
