@@ -24,6 +24,9 @@
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
 ##
+"""
+EasyBuild support for goalf compiler toolchain (includes GCC, OpenMPI, ATLAS, BLACS, ScaLAPACK and FFTW).
+"""
 
 from easybuild.toolchains.compiler.gcc import Gcc
 from easybuild.toolchains.fft.fftw import Fftw
@@ -34,4 +37,5 @@ from easybuild.toolchains.mpi.openmpi import OpenMPI
 
 
 class Goalf(Gcc, OpenMPI, Atlas, Blacs, ScaLAPACK, Fftw):
+    """Compiler toolchain with GCC, OpenMPI, ATLAS, BLACS, ScaLAPACK and FFTW."""
     NAME = 'goalf'
