@@ -23,10 +23,14 @@
 # You should have received a copy of the GNU General Public License
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
 ##
+"""
+EasyBuild support for gompi compiler toolchain (includes GCC and OpenMPI).
+"""
 
 from easybuild.toolchains.compiler.gcc import Gcc
 from easybuild.toolchains.mpi.openmpi import OpenMPI
 
 
 class Gompi(Gcc, OpenMPI):
+    """Compiler toolchain with GCC and OpenMPI."""
     NAME = 'gompi'
