@@ -2,6 +2,7 @@
 # Copyright 2012 Ghent University
 # Copyright 2012 Stijn De Weirdt
 # Copyright 2012 Kenneth Hoste
+# Copyright 2012 Jens Timmerman
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -31,14 +32,15 @@ Support for MPICH2 as toolchain MPI library.
 from easybuild.tools.toolchain.mpi import Mpi
 from easybuild.tools.variables import CommandFlagList
 
-
 TC_CONSTANT_MPICH2 = "MPICH2"
+TC_CONSTANT_MPI_TYPE_MPICH = "MPI_TYPE_MPICH"
 
 
 class Mpich2(Mpi):
     """MPICH2 MPI class"""
     MPI_MODULE_NAME = ["MPICH2"]
     MPI_FAMILY = TC_CONSTANT_MPICH2
+    MPI_TYPE = TC_CONSTANT_MPI_TYPE_MPICH
 
     MPI_LIBRARY_NAME = 'mpich'
 
@@ -51,7 +53,7 @@ class Mpich2(Mpi):
                              }
 
     def _set_mpi_compiler_variables(self):
-        """Add MPICH_XXX variables to set."""
+        """Set the MPICH_{CC, CXX, F77, F90} variables."""
 
         # this needs to be done first, otherwise e.g., CC is set to MPICC if the usempi toolchain option is enabled
         for var in ["CC", "CXX", "F77", "F90"]:
