@@ -51,10 +51,16 @@ from easybuild.tools.build_log import get_log
 log = get_log('fileTools')
 errorsFoundInLog = 0
 
-strictness = 'warn'
+# constants for strictness levels
+IGNORE = 'ignore'
+WARN = 'warn'
+ERROR = 'error'
+
+# default strictness level
+strictness = WARN
 
 
-def extract_file(fn, dest, extra_options=None, overwrite=False):
+def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
     """
     Given filename fn, try to extract in directory dest
     - returns the directory name in case of success
@@ -79,7 +85,11 @@ def extract_file(fn, dest, extra_options=None, overwrite=False):
     except OSError, err:
         log.error("Can't change to directory %s: %s" % (absDest, err))
 
-    cmd = extract_cmd(fn, overwrite=overwrite)
+    if not cmd:
+        cmd = extract_cmd(fn, overwrite=overwrite)
+    else:
+        # complete command template with filename
+        cmd = cmd % fn
     if not cmd:
         log.error("Can't extract file %s with unknown filetype" % fn)
 
@@ -301,7 +311,28 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
 
     return result
 
+def adjust_cmd(func):
+    """Make adjustments to given command, if required."""
 
+    def inner(cmd, *args, **kwargs):
+        # SuSE hack
+        # - profile is not resourced, and functions (e.g. module) is not inherited
+        if 'PROFILEREAD' in os.environ and (len(os.environ['PROFILEREAD']) > 0):
+            filepaths = ['/etc/profile.d/modules.sh']
+            extra = ''
+            for fp in filepaths:
+                if os.path.exists(fp):
+                    extra = ". %s &&%s" % (fp, extra)
+                else:
+                    log.warning("Can't find file %s" % fil)
+
+            cmd = "%s %s" % (extra, cmd)
+
+        return func(cmd, *args, **kwargs)
+
+    return inner
+
+@adjust_cmd
 def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True, log_output=False, path=None):
     """
     Executes a command cmd
@@ -330,18 +361,6 @@ def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True
     else:
         runLog = None
 
-    # SuSE hack
-    # - profile is not resourced, and functions (e.g. module) is not inherited
-    if 'PROFILEREAD' in os.environ and (len(os.environ['PROFILEREAD']) > 0):
-        files = ['/etc/profile.d/modules.sh']
-        extra = ''
-        for fil in files:
-            if not os.path.exists(fil):
-                log.error("Can't find file %s" % fil)
-            extra = ". %s && " % fil
-
-        cmd = "%s %s" % (extra, cmd)
-
     readSize = 1024 * 8
 
     try:
@@ -380,6 +399,7 @@ def run_cmd(cmd, log_ok=True, log_all=False, simple=False, inp=None, regexp=True
     return parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp)
 
 
+@adjust_cmd
 def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, regexp=True, std_qa=None, path=None):
     """
     Executes a command cmd
@@ -396,23 +416,10 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
         if path:
             os.chdir(path)
 
-        log.debug("runQandA: running cmd %s (in %s)" % (cmd, os.getcwd()))
+        log.debug("run_cmd_qa: running cmd %s (in %s)" % (cmd, os.getcwd()))
     except:
         log.info("running cmd %s in non-existing directory, might fail!" % cmd)
 
-
-    # SuSE hack
-    # - profile is not resourced, and functions (e.g. module) is not inherited
-    if 'PROFILEREAD' in os.environ and (len(os.environ['PROFILEREAD']) > 0):
-        files = ['/etc/profile.d/modules.sh']
-        extra = ''
-        for fil in files:
-            if not os.path.exists(fil):
-                log.error("Can't find file %s" % fil)
-            extra = ". %s && " % fil
-
-        cmd = "%s %s" % (extra, cmd)
-
     # Part 1: process the QandA dictionary
     # given initial set of Q and A (in dict), return dict of reg. exp. and A
     #
@@ -480,7 +487,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
     try:
         p = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, stdin=PIPE, close_fds=True, executable="/bin/bash")
     except OSError, err:
-        log.error("runQandA init cmd %s failed:%s" % (cmd, err))
+        log.error("run_cmd_qa init cmd %s failed:%s" % (cmd, err))
 
     # initial short sleep
     time.sleep(0.1)
@@ -499,7 +506,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             stdoutErr += tmpOut
         # recv_some may throw Exception
         except (IOError, Exception), err:
-            log.debug("runQandA cmd %s: read failed: %s" % (cmd, err))
+            log.debug("run_cmd_qa cmd %s: read failed: %s" % (cmd, err))
             tmpOut = None
 
         hit = False
@@ -507,7 +514,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
             res = q.search(stdoutErr)
             if tmpOut and res:
                 fa = a % res.groupdict()
-                log.debug("runQandA answer %s question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
+                log.debug("run_cmd_qa answer %s question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
                 send_all(p, fa)
                 hit = True
                 break
@@ -516,7 +523,7 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
                 res = q.search(stdoutErr)
                 if tmpOut and res:
                     fa = a % res.groupdict()
-                    log.debug("runQandA answer %s standard question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
+                    log.debug("run_cmd_qa answer %s standard question %s out %s" % (fa, q.pattern, stdoutErr[-50:]))
                     send_all(p, fa)
                     hit = True
                     break
@@ -542,9 +549,9 @@ def run_cmd_qa(cmd, qa, no_qa=None, log_ok=True, log_all=False, simple=False, re
                 os.killpg(p.pid, signal.SIGKILL)
                 os.kill(p.pid, signal.SIGKILL)
             except OSError, err:
-                log.debug("runQandA exception caught when killing child process: %s" % err)
-            log.debug("runQandA: full stdouterr: %s" % stdoutErr)
-            log.error("runQandA: cmd %s : Max nohits %s reached: end of output %s" % (cmd,
+                log.debug("run_cmd_qa exception caught when killing child process: %s" % err)
+            log.debug("run_cmd_qa: full stdouterr: %s" % stdoutErr)
+            log.error("run_cmd_qa: cmd %s : Max nohits %s reached: end of output %s" % (cmd,
                                                                                     maxHitCount,
                                                                                     stdoutErr[-500:]
                                                                                     ))
@@ -572,13 +579,13 @@ def parse_cmd_output(cmd, stdouterr, ec, simple, log_all, log_ok, regexp):
     """
     will parse and perform error checks based on strictness setting
     """
-    if strictness == 'ignore':
+    if strictness == IGNORE:
         check_ec = False
         use_regexp = False
-    elif strictness == 'warn':
+    elif strictness == WARN:
         check_ec = True
         use_regexp = False
-    elif strictness == 'error':
+    elif strictness == ERROR:
         check_ec = True
         use_regexp = True
     else:
