@@ -53,9 +53,17 @@ class Modules(object):
     Interact with modules.
     """
     def __init__(self, modulePath=None):
+        """
+        Create a Modules object
+        @param modulePath: A list of paths where the modules can be located
+        @type modulePath: list
+        """
         self.log = get_log(self.__class__.__name__)
-
-        self.modulePath = modulePath
+        # make sure we don't have the same path twice
+        if modulePath:
+            self.modulePath = set(modulePath)
+        else:
+            self.modulePath = None
         self.modules = []
 
         self.check_module_path()
@@ -158,6 +166,20 @@ class Modules(object):
         for mod in self.modules:
             self.run_module('load', "/".join(mod))
 
+    def unload(self):
+        """
+        Unload all requested modules.
+        """
+        for mod in self.modules:
+            self.run_module('unload', "/".join(mod))
+
+    def purge(self):
+        """
+        Purge loaded modules.
+        """
+        self.log.debug("List of loaded modules before purge: %s" % os.getenv('_LMFILES_'))
+        self.run_module('purge', '')
+
     def show(self, name, version):
         """
         Run 'module show' for the specified module.
@@ -173,6 +195,8 @@ class Modules(object):
         else:
             modinfo = self.show(name, version)
 
+            self.log.debug("modinfo (split): %s" % modinfo.split('\n'))
+
             # second line of module show output contains full path of module file
             return modinfo.split('\n')[1].replace(':', '')
 
@@ -188,7 +212,9 @@ class Modules(object):
         originalModulePath = os.environ['MODULEPATH']
         if kwargs.get('modulePath', None):
             os.environ['MODULEPATH'] = kwargs.get('modulePath')
+        self.log.debug('Current MODULEPATH: %s' % os.environ['MODULEPATH'])
 
+        self.log.debug("Running 'modulecmd python %s' from %s..." % (' '.join(args), os.getcwd()))
         proc = subprocess.Popen(['modulecmd', 'python'] + args,
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         # stdout will contain python code (to change environment etc)
